//
//  GPSController.m
//  GPS
//
//  Created by Phidgets on 2015-07-16.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "GPSController.h"

@implementation GPSController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Stop all events before closing
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetGPS_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    
    result = PhidgetGPS_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to create gps channel"];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        const char* resultString;
        Phidget_getErrorDescription(result, &resultString);
        NSString *errorString = [NSString stringWithFormat:@"Failed to initialize channel: %@",  [NSString stringWithUTF8String:resultString]];
        [self addError:errorString];
    }
    
    result = PhidgetGPS_setOnPositionChangeHandler(ch, gotPositionChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign position change handler"];
    }
    
    result = PhidgetGPS_setOnPositionFixStateChangeHandler(ch, gotPositionFixStateChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign frequency change handler"];
    }
    
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
    formatter = [[NSDateComponentsFormatter alloc]init];
    formatter.unitsStyle = NSDateComponentsFormatterUnitsStylePositional;
    formatter.allowedUnits = NSCalendarUnitHour | NSCalendarUnitMinute | NSCalendarUnitSecond;
    formatter.zeroFormattingBehavior = NSDateComponentsFormatterZeroFormattingBehaviorNone;
    timer = [NSTimer scheduledTimerWithTimeInterval:0.1
                                             target:self
                                           selector:@selector(timerFireMethod:)
                                           userInfo:nil
                                            repeats:YES];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign attach handler"];
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign detach handler"];
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign error handler"];
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

- (void)timerFireMethod:(NSTimer *)timer{
    PhidgetGPS_Time time;
    PhidgetGPS_Date date;
    
    if(!PhidgetGPS_getTime(ch, &time) && !PhidgetGPS_getDate(ch, &date))
    {
        [timeLabel setStringValue:[NSString stringWithFormat:@"%02d:%02d:%02d",
                                         time.tm_hour,time.tm_min,time.tm_sec]];
        [dateLabel setStringValue:[NSString stringWithFormat:@"%02d/%02d/%02d",date.tm_mday,date.tm_mon,date.tm_year]];
    }
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

static void gotPositionChangeData(PhidgetGPSHandle phid, void *context, double latitude, double longitude, double altitude){
        [(__bridge id)context performSelectorOnMainThread:@selector(onPositionChangeHandler:)
                                               withObject:[NSArray arrayWithObjects:[NSNumber numberWithDouble:latitude], [NSNumber numberWithDouble:longitude],[NSNumber numberWithDouble:altitude], nil]
                                            waitUntilDone:NO];
}

static void gotPositionFixStateChangeData(PhidgetGPSHandle phid, void *context, int status){
        [(__bridge id)context performSelectorOnMainThread:@selector(onPositionFixStateChangeHandler:)
                                               withObject:[NSNumber numberWithInt:status]
                                            waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [dataBox setHidden:NO];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Reset
    [timeLabel setStringValue:@""];
    [dateLabel setStringValue:@""];
    [altitudeText setStringValue:@""];
    [longitudeText setStringValue:@""];
    [latitudeText setStringValue:@""];
    [velocityText setStringValue:@""];
    [headingText setStringValue:@""];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void)errorHandler:(NSArray *)errorEventData{
    [self addError:[errorEventData objectAtIndex:1]];
}

-(void)addError:(NSString *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",errorString]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

- (void) onPositionChangeHandler:(NSArray *)gpsArray{
    double velocity,heading;
    double latitude = [[gpsArray objectAtIndex:0] doubleValue];
    double longitude = [[gpsArray objectAtIndex:1]doubleValue];
    double altitude = [[gpsArray objectAtIndex:2]doubleValue];
    [latitudeText setStringValue:[NSString stringWithFormat:@"%0.3fº",latitude]];
    [longitudeText setStringValue:[NSString stringWithFormat:@"%0.3fº",longitude]];
    [altitudeText setStringValue:[NSString stringWithFormat:@"%0.3f m",altitude]];
    
    PhidgetGPS_getVelocity(ch, &velocity);
    PhidgetGPS_getHeading(ch, &heading);
    
    [velocityText setStringValue:[NSString stringWithFormat:@"%0.1f km/h",velocity]];
    [headingText setStringValue:[NSString stringWithFormat:@"%0.1fº",heading]];
}

- (void)onPositionFixStateChangeHandler:(NSNumber *)status{
    [fixAquiredCheck setIntValue:[status intValue]];
    if(![fixAquiredCheck intValue]){
        [latitudeText setStringValue:@""];
        [longitudeText setStringValue:@""];
        [altitudeText setStringValue:@""];
        [headingText setStringValue:@""];
        [velocityText setStringValue:@""];
    }
}

- (IBAction)unwantedClick:(id)sender{
    [fixAquiredCheck setIntValue:(![fixAquiredCheck state])];
}

@end
